/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        20.03.2014
 Description:    DMX_4 Kanal_Controller


 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>
#include <avr/delay.h>

#define DMX_BAUD 250000
#define DMX_LOST_TIMEOUT 500

#define LED_R_PIN_OUT DDRD |= (1<<PD5);
#define LED_G_PIN_OUT DDRD |= (1<<PD6);
#define LED_B_PIN_OUT DDRB |= (1<<PB1);
#define LED_W_PIN_OUT DDRB |= (1<<PB2);

#define PWM_REGISTER_B	OCR1A
#define PWM_REGISTER_W	OCR1B
#define PWM_REGISTER_G 	OCR0A
#define PWM_REGISTER_R	OCR0B

volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;

volatile unsigned int dmx_adresse = 0;
volatile unsigned char dmx_buffer[5];

//############################################################################
//DMX Senderoutine
ISR (USART_RX_vect)
//############################################################################
{
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	unsigned char tmp = 0;
	
	tmp =  UDR0;
	
	if(UCSR0A&(1<<FE0))
	{
		if(dmx_channel_rx_count > 1)
		{
			dmx_lost = 0;
		}
		dmx_channel_rx_count = 0;	
		dmx_buffer[0] = tmp;
		if(dmx_buffer[0] == 0)
		{
			dmx_valid = 1;
			dmx_channel_rx_count++;
		}
		else
		{
			dmx_valid = 0;
		}
		return;
	}
	
	if(dmx_valid)
	{
		if(dmx_channel_rx_count == dmx_adresse) dmx_buffer[1] = tmp;
		if(dmx_channel_rx_count == dmx_adresse+1) dmx_buffer[2] = tmp;
		if(dmx_channel_rx_count == dmx_adresse+2) dmx_buffer[3] = tmp;
		if(dmx_channel_rx_count == dmx_adresse+3) dmx_buffer[4] = tmp;
		
		if(dmx_channel_rx_count < 514)
		{
			dmx_channel_rx_count++;
		}
		return;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER2_COMPA_vect)
//############################################################################
{
	if(dmx_lost<DMX_LOST_TIMEOUT)
	{
		dmx_lost++;
	}
}

//############################################################################
//Hauptprogramm
int main (void) 
//############################################################################
{  
	unsigned int dmx_adresse_tmp;

	//Pullup
	PORTC |= 0xFF;
	PORTD |= (1<<PD3|1<<PD4|1<<PD7);
	PORTB |= (1<<PB0);

	//Init usart DMX-BUS RX
	UBRR0H   = (unsigned char)(F_CPU / (DMX_BAUD * 16L) - 1)>>8;
	UBRR0L   = (unsigned char)(F_CPU / (DMX_BAUD * 16L) - 1);
	UCSR0B|=(1 << RXEN0 | 1<< RXCIE0);
	UCSR0C|=(1<<USBS0); //USBS0 2 Stop bits	
		
	//Timer2 Timecounter fr DMX Ausfall
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS22|1<<CS21|1<<CS20);
	TIMSK2 |= (1<<OCIE2A);
	OCR2A = F_CPU/1024/1000 - 1; //Tick
	
	//Globale Interrupts Enable
	sei();
	
	//DIR PIN RX Data
	DDRD |=(1<<PD2);
	PORTD &=~(1<<PD2);
	
	//PWM Ports auf Output
	LED_R_PIN_OUT;
	LED_G_PIN_OUT;
	LED_B_PIN_OUT;
	LED_W_PIN_OUT;

	//PWM Timer 0
	TCCR0A |= (1<<COM0A1)|(1<<COM0A0)|(1<<COM0B1)|(1<<COM0B0)|(1<<WGM00)|(1<<WGM01);
	TCCR0B |= (1<<CS01)|(1<<CS00);
	//PWM Timer 1
	TCCR1A |= (1<<COM1A1)|(1<<COM1A0)|(1<<COM1B1)|(1<<COM1B0)|(1<<WGM10);
	TCCR1B |= (1<<WGM12)|(1<<CS11)|(1<<CS10);
	
	//ROT an	
	PWM_REGISTER_R = 0;
	PWM_REGISTER_G = 255;
	PWM_REGISTER_B = 255;
	PWM_REGISTER_W = 255;
	_delay_ms(500);
	
	//GRN an
	PWM_REGISTER_R = 255;
	PWM_REGISTER_G = 0;
	_delay_ms(500);
	
	//BLAU an
	PWM_REGISTER_G = 255;
	PWM_REGISTER_B = 0;
	_delay_ms(500);
	
	//WEI an
	PWM_REGISTER_B = 255;
	PWM_REGISTER_W = 0;
	_delay_ms(500);
	PWM_REGISTER_W = 255;
	
	//Endlosschleife
	while(1)
	{
		dmx_adresse_tmp = 0;
		if(!(PIND&(1<<PD4))) dmx_adresse_tmp |= 0x01;		
		if(!(PIND&(1<<PD3))) dmx_adresse_tmp |= 0x02;
 		if(!(PINC&(1<<PC5))) dmx_adresse_tmp |= 0x04;
		if(!(PINC&(1<<PC4))) dmx_adresse_tmp |= 0x08;
		if(!(PINC&(1<<PC3))) dmx_adresse_tmp |= 0x10;
		if(!(PINC&(1<<PC2))) dmx_adresse_tmp |= 0x20;
		if(!(PINC&(1<<PC1))) dmx_adresse_tmp |= 0x40;
		if(!(PINC&(1<<PC0))) dmx_adresse_tmp |= 0x80;
		if(!(PINB&(1<<PB0))) dmx_adresse_tmp |= 0x0100;
		//if(!(PIND&(1<<PD7))){};
		
		if(dmx_adresse != dmx_adresse_tmp) dmx_adresse =  dmx_adresse_tmp;

			PWM_REGISTER_R = 255 - dmx_buffer[1];
			PWM_REGISTER_G = 255 - dmx_buffer[2];
			PWM_REGISTER_B = 255 - dmx_buffer[3];
			PWM_REGISTER_W = 255 - dmx_buffer[4];

		
		if(dmx_lost==DMX_LOST_TIMEOUT)
		{
			dmx_buffer[1] = 255;
			dmx_buffer[2] = 0;
			dmx_buffer[3] = 0;
			dmx_buffer[4] = 0;
		}
	}
}

